function estimates=coef_bma(bmao, exact, order_by_pip, include_constant, incl_possign, std_coefs, condi_coef)
%Posterior Inclusion Probabilities and Coefficients from a 'bma' Object
%
%estimates=coef_bma(bmao, exact, order_by_pip, include_constant, incl_possign, std_coefs, condi_coef)
%
% Note that default inputs may be obtained by passing an empty value (e.g. '') or NaN
%% INPUTS:
%required inputs:
% * bmao: a 'bma' object (as e.g. resulting from bms)
%optional inputs:
% * exact: if exact==false (default), then PIPs, coefficents, etc. will be based 
%        on aggregate information from the sampling chain with posterior model 
%        distributions based on MCMC frequencies (except in case of enumeration - cf. 'Notes');
%        if exact==true, estimates will be based on the nmodel best models encountered 
%        by the sampling chain, with the posterior model distribution based on their 
%        exact marginal likelihoods - cf. 'Notes' below.
% * order_by_pip: order_by_pip==true (default) orders the resulting matrix according
%        to posterior inclusion probabilites, order.by.pip=FALSE ranks them according 
%        to the original data (order of the covariates as in provided in X_data to bms)
% * include_constant: If include_constant===true then the
%        resulting matrix includes the expected value of the constant in its last
%        row. Default false
% * incl_possign: If incl.possign== false, then the sign probabilites 
%        column (cf. 'Output' below) is omitted from the result. Default true 
% * std_coefs: If std_coefs==true then the expected values and
%        standard deviations are returned in standardized form, i.e. as if the
%        original data all had mean zero and variance 1. If std_coefs==false
%        (default) then both expected values and standard deviations are returned 'as is'
% * condi_coef: If condi_coef==false (default) then coefficients ?_i and
%        standard deviations are unconditional posterior expected values, as in
%        standard model averaging; if condi.coef=FALSE then they are given as conditional on 
%        inclusion (equivalent to ?_i / PIP_i)  
%
%% OUTPUT:
% A matrix with five columns (or four if incl_possign==false)
% * 1st Column 'PIP': Posterior inclusion probabilities ? p(?|i \in ?, Y) / sum p(?|Y)
% * 2nd Column 'Post Mean': posterior expected value of coefficients, unconditional 
%        E(?|Y)=? p(?|Y) E(?|?,Y), where E(?_i|?,i \notin ?, Y)=0 if condi.coef=FALSE, or 
%        conditional on inclusion (E(?|Y) / ? p(?|Y, i \in ?) ) if condi.coef=TRUE
% * 3rd Column 'Post SD': posterior standard deviation of coefficients,
%        unconditional or conditional on inclusion, depending on condi.coef
% * 4th Column 'Cond.Pos.Sign': The ratio of how often the coefficients'
%        expected values were positive conditional on inclusion. (over all visited
%        models in case exact=FALSE, over the best models in case exact=TRUE)  
% * 5th Column 'Idx': the original order of covariates as the were used for
%        sampling. (if included, the constant has index 0) 
% If no output is requested, then coef_bma prints the above matrix out too
% the console in a nicely formatted fashion
%
%
%% NOTES: 
% In case the argument exact==true, the PIPs, coefficent statistics and
% conditional sign probabilities are computed on the basis of the (500)
% best models the sampling chain encoutered (cf. argument nmodel  in bms).
% Here, the weights for Bayesian model averaging (BMA) are the posterior
% marginal likelihoods of these best models.    
% In case exact==false, then these statistics are based on all accepted
% models (except burn-ins): If mcmc="enumerate" then this are simply all
% models of the traversed model space, with their marginal likelihoods
% providing the weights for BMA.   
% If, however, the bma object bmao was based on an MCMC sampler (e.g. when
% bms argument mcmc="bd"), then BMA statistics are computed differently: In
% contrast to above, the weights for BMA are MCMC frequencies, i.e. how
% often the respective models were encountered by the MCMC sampler 
% (cf. a comparison of MCMC frequencies and marginal likelihoods for the 
% best models via the function pmp_bma)     
%
%% EXAMPLE:
% load datafls;
% mm=bms(datafls);
% %standard BMA PIPs and coefficients from the MCMC sampling chain, based on how frequently th emodels were drawn
% coef_bma(mm);
%
% %standardized coefficents, ordered by index
% coef_bma(mm,'',false,'','',true);
% 
% %coefficents conditional on inclusion:
%coef_bma(mm,[],[],[],[],[],true)
% %same as
%ests=coef_bma(mm);
%ests(:,2)./ests(:,1)
%
% %PIPs, coefficients, and signs based on the best 200 models
%coef_bma(mm,true)
%________________________________________________
%
%% SEE ALSO
% bms, summary_bma, pmp_bma
% 
% Equivalent function in R: coef.bma
% Author: Stefan Zeugner 2010 
% Url: http://bms.zeugner.eu


%input checks
    is_bma(bmao,false);
    
if nargin<2; exact=NaN; end; if isempty(exact); exact=NaN; end;
if nargin<3; order_by_pip=NaN; end; if isempty(order_by_pip); order_by_pip=NaN; end;
if nargin<4; include_constant=NaN; end; if isempty(include_constant); include_constant=NaN; end;
if nargin<5; incl_possign=NaN; end; if isempty(incl_possign); incl_possign=NaN; end;
if nargin<6; std_coefs=NaN; end; if isempty(std_coefs); std_coefs=NaN; end;    
if nargin<7; condi_coef=NaN; end; if isempty(condi_coef); condi_coef=NaN; end;        
    
exact=test4bool('exact');  
order_by_pip=test4bool('order_by_pip');  
include_constant=test4bool('include_constant');  
incl_possign=test4bool('incl_possign');  
std_coefs=test4bool('std_coefs');  
condi_coef=test4bool('condi_coef');  


% construct the R command
coefcmd = ['bmstmp=estimates.bma(bma' bmao.suffix ];
if ~isnan(exact); coefcmd = [coefcmd ' ,exact=' num2str(exact)]; end;
if ~isnan(order_by_pip); coefcmd = [coefcmd ' ,order.by.pip=' num2str(order_by_pip)]; end;
if ~isnan(include_constant); coefcmd = [coefcmd ' ,include.constant=' num2str(include_constant)]; end;
if ~isnan(incl_possign); coefcmd = [coefcmd ' ,incl.possign=' num2str(incl_possign)]; end;
if ~isnan(std_coefs); coefcmd = [coefcmd ' ,std.coefs=' num2str(std_coefs)]; end;    
if ~isnan(condi_coef); coefcmd = [coefcmd ' ,condi.coef=' num2str(condi_coef)]; end;    
coefcmd = [coefcmd ')'];

%evaluate and get result
evalinR(coefcmd);
estimates2=rcell2mat(getRdata('bmstmp'));



if nargout==0;
% if user just wants results on screen, print them out nicely
     lLeadBlanks=4;
     coefcontainer=c2cstr(estimates2);
     coefcontainer = vertcat({'PIP','Post Mean','Post SD','Cond.Pos.Sign','Idx'},coefcontainer); 
     coefcontainer=horzcat(bmao.data.Properties.VarNames([0;bmao.coef(:,end)]+1),coefcontainer);
     coefcontainer{1,1}='Variable';
     disp(Cstr2format(coefcontainer,4,0,lLeadBlanks,[-1;1;1;1;1;1],0));
 else
     estimates=estimates2;
 end
 
function boolout = test4bool(boolname)
    mybool=evalin('caller',boolname);
    if any(isnan(mybool))
        boolout=NaN;
        return;
    end
    try 
        boolout=logical(mybool(1)); 
    catch 
        error(['argument ''' boolname ''' must be logical']); 
    end