function denscell = density_bma(bmao, reg, std_coefs, n, doplot)
%Coefficient Marginal Posterior Densities
%
%denscell = density_bma(bmao, reg, std_coefs, n, doplot)
%
% Note that default inputs may be obtained by passing an empty value (e.g. '') or NaN
%% INPUTS:
%required inputs:
% * bmao: a 'bma' object (as e.g. resulting from bms)
%optional inputs:
% * reg: A scalar integer vector detailing which covariate's
%       coefficient should be plotted. If reg=NaN (default), then all regressors
%       are plotted one after the other, waiting for user interaction.  
% * std_coefs: logical. If true then the posterior density is estimated 
%       for standardized coefficients (representing the case where all variables
%       have mean zero and standard deviation 1) - default is false. 
% * n: numeric. the number of equally spaced points at which the density is
%       to be estimated. default is n==300
% * doplot: logical. If doplot==true (default), the density is plotted; if doplot==
%       false then density_bma only returns the estimated posterior densities
%       without plotting.   
%
%% OUTPUT:
%   only returned if an output element is requested:
%    a cell array with as many elements as bmao has covariates. Each element
%    (if contained in reg) contains a 2 column matrix with the first column
%    indicating the x-axis, and the 2nbd column the y-axis for a density
%    plot;
%
%
%% NOTES: 
%  The computed marginal posterior densities from density.bma are a
%  Bayesian Model Averaging mixture of the marginal posterior densities of
%  the individual models. The accuracy of the result therefore depends on
%  the number of 'best' models contained in x (cf. argument nmodel in bms).   
%
%  The marginal posterior density can be interpreted as 'conditional on
%  inclusion': If the posterior inclusion probability of a variable is
%  smaller than one, then some of its posterior density is Dirac at zero.
%  Therefore the integral of the returned density vector adds up to the
%  posterior inclusion probability, i.e. the probability that the
%  coefficient is not zero.      
% 
%  Note that the vertical line plotted in density plots indicates the
%  conditional expected value of the cooefficient
%
%% EXAMPLE:
% load datafls;
% mm=bms(datafls);
% density_bma(mm,reg=12); % plotting one
% density_bma(mm,reg=1:3); % plotting three
%
% %just calculating:
% dd_list=density_bma(mm,1:3,[],400,false);
% plot(dd_list{1}(:,1),dd_list{1}(:,2));
%________________________________________________
%
%% SEE ALSO
% bms, summary_bma, pmp_bma
% 
% Equivalent function in R: density.bma
% Author: Stefan Zeugner 2010 
% Url: http://bms.zeugner.eu





  is_bma(bmao,false);

  if nargin<2; reg=NaN; end; if isempty(reg); reg=NaN; end
  if nargin<3; std_coefs=NaN; end; if isempty(std_coefs); std_coefs=NaN; end
  if nargin<4; n=NaN; end; if isempty(n); n=NaN; end
  if nargin<5; doplot=NaN; end; if isempty(doplot); doplot=NaN; end
  
  if isnan(std_coefs); std_coefs=false; end;
  if isnan(n); n=300; end;
  if isnan(reg); reg=1:size(bmao.coef,1); end;
  if isnan(doplot); doplot=true; end;
   
  K=size(bmao.coef,1);
  errmsg=['reg must be vector or scalar of integers between 0 and ' num2str(K)];
  if ~isnumeric(reg); error(ermsg); end;
  reg=floor(reg);
  if any(reg<1)| any(reg>K); error(ermsg); end;
  
  try 
    std_coefs=logical(std_coefs(1));    
  catch
      error('std_coefs must be logical scalar');
  end
  
  try 
    doplot=logical(doplot(1));    
  catch
      error('doplot must be logical scalar');
  end
  
  pips=sortrows(bmao.coef,5); pips=pips(:,1);
  vblnames=c2cstr(reg);
  try
      vblnames = bmao.data.Properties.VarNames;
      vblnames = vblnames(:); vblnames=vblnames(2:end);
  end;
  
  outcell=cell(1,length(pips));  

  if (length(reg)>1) && doplot ; disp('click or press key for next density plot...'); end
  for i=reg
     outcell{i}=singledensity(bmao, i, std_coefs,n);     
     if doplot; 
         if size(outcell{i},1)==1; 
              warning(['no density plot for variable ' num2str(i) ' produced']); 
          else
             plot(outcell{i}(:,1),outcell{i}(:,2));
             
             title(['Marginal Density: Variable ''', vblnames{i},  ''' (PIP ', num2str(round(pips(i)*10000)/100), '%)']);
             xlabel('Coefficient'); ylabel('Density');
             condev=sum(outcell{i}(:,1).*outcell{i}(:,2)/sum(outcell{i}(:,2)));
              line([condev,condev],[0,outcell{i}(sum(outcell{i}(:,1)<condev)+1,2)]);
             if i~=reg(end);  waitforbuttonpress ; end
         end
     end     
    
  end
  
  if nargout>0;   denscell=outcell; end;
  
function densmat = singledensity(bmao, i, std_coefs, n)
    evalinR(['bmstmp=densitybma_single_matlab(bma' bmao.suffix ', reg=' num2str(i) ', std.coefs=' num2str(std_coefs) ', n=' num2str(n) ');']);
    densmat=getRdata('bmstmp');
    
    