function signmatrix=image_bma(bmao,yprop2pip,order_by_pip,indexofmodels)
% Plots coefficient signs vs. model probabilities for the best models
%
%function signmatrix=image_bma(bmao,yprop2pip,order_by_pip,indexofmodels)
%
% Note that default inputs may be obtained by passing an empty value (e.g. '') or NaN
%% INPUTS:
% * bmao: a 'bma' object (as e.g. resulting from bms)
% * yprop2pip: if yprop2pip==true then the grid lines on the vertical axis are 
%         scaled according to the coefficients' inclusion probabilites.
%         If yprop2pip=FALSE (default) then the gridlines on the vertical axis are equidistant.
% * order_by_pip: with order.by.pip==true (default), coefficents are sorted according to their 
%         posterior inclusion probabilites along the vertical axis. If order.by.pip=FALSE they 
%         are ordered as they were provided to bms.
% * indexofmodels: an integer vector index indicating which of the best
%         models to take into accoount - default is alll best models stored
%         in bmao
%
%% OUTPUT:
%   If an output argument is requested, then image_bma returns a matrix
%   indicating with signs for the the required models (conforms to 
%   sign(beta_draws_bma(bmao)) if order_by_pip===false)
%
%% NOTES: 
% Under default settings, blue corresponds to positive sign, red to a negative sign, white to non-inclusion. 
% image_bma is mainly a call to the Matlab function pcolor, therefore the
% same settings apply
%
%% EXAMPLE:
% load datafls;
% mm=bms(datafls);
% image_bma(mm) 
% image_bma(mm,'','',1:50) %plot only for the best 50 models
% image_bma(mm,true);   %make y axis proportional to PIPs
% shading('flat'); % remove grid lines
%
%% SEE ALSO
% bms, beta_draws_bma, pmp_bma
% 
% Equivalent function in R: image.bma
% Author: Stefan Zeugner 2010 
% Url: http://bms.zeugner.eu


% input checks
is_bma(bmao,false);
if nargin<2; yprop2pip=NaN; end;
if nargin<3; order_by_pip=NaN; end;
if nargin<4; indexofmodels=NaN; end;

if isempty(yprop2pip); yprop2pip=NaN; end;
if isempty(order_by_pip); order_by_pip=NaN; end;
if isempty(indexofmodels); indexofmodels=NaN; end;

if isnan(yprop2pip); yprop2pip=false; end;
if isnan(order_by_pip); order_by_pip=true; end;

try yprop2pip=logical(yprop2pip(1)); catch error('yprop2pip must be logical scalar'); end;
try order_by_pip=logical(order_by_pip(1)); catch error('order_by_pip must be logical scalar'); end;
if ~isnan(indexofmodels); 
    try
      indexofmodels=indexofmodels(:);
      indexofmodels=round(indexofmodels);
      indexofmodels= sort(unique(indexofmodels));
     catch
      error('indexofmodels must be a vector of integers')
     end
 end



 %get the computative quantities
ests=coef_bma(bmao, true, order_by_pip);
betas=beta_draws_bma(bmao);
pmps_res=pmp_bma(bmao);
if isnan(indexofmodels); indexofmodels = 1:size(pmps_res,1); end;

normali_factor = sum(pmps_res(indexofmodels, 2));
pmps=pmps_res(indexofmodels,1)*normali_factor;
betasign=sign(betas);
pips=ests(:,1);
idx=ests(:,end);
betasign=betasign(idx,indexofmodels);

if nargout>0 ; signmatrix=betasign; end;


% prepare plotting

betasign_traf=betasign(length(pips):-1:1,:);
betasign_traf=vertcat(betasign_traf, repmat(0,1,size(betasign_traf,2)));
betasign_traf=horzcat(betasign_traf, repmat(0,size(betasign_traf,1),1));

colormap([1,0,0; 1,1,1; 0,0,1]);
cpmps=cumsum(pmps); cpmps=[0; cpmps];
cpips=1:(length(pips)+1);
if yprop2pip; 
   cpips=vertcat(0,cumsum(pips(end:-1:1)));
   cpips=cpips./cpips(end).*(length(cpips)-1)+1;
end;
pcolor(cpmps,cpips,betasign_traf);

title(['Model Inclusion Based on Best ' num2str(length(pmps)) ' Models']);
xlabel('Cumulative Model Probabilities');
mylabels=char(bmao.data.Properties.VarNames); mylabels=mylabels(2:end,:);
mylabels=mylabels(idx,:); 
mylabels=mylabels(end:-1:1,:);
mylabels=vertcat(repmat(' ',1,size(mylabels,2)),mylabels);
if size(mylabels,2)>8; mylabels=mylabels(:,1:8); end;
set(gca,'YTick',cpips);
set(gca,'YTickLabel',mylabels);
%set(gca,'FontSize',max(get(gca,'FontSize')*0.8,5));


