function pmps = pmp_bma(bmao, index_of_models)
%Posterior Model Probabilities under MCMC and likelihood-based concepts
%
%pmps = pmp_bma(bmao)
%
%% INPUTS:
% * bmao: a 'bma' object (as e.g. resulting from bms)
% * index_of_models (optional): integer index of models to calculate (e.g.
%      1:3 for the three best models)
%
%% OUTPUT:
% the result is a matrix, with its rows describing the best models by
%    decreasing importance 
% There are two columns in the matrix:
% 1st column: PMP (Exact) 	posterior model probabilities based on the
%       posterior likelihoods of the best models in bmao, relative to all 'best' models 
% 2nd column: PMP (MCMC) 	posterior model probabilities of the best
%    models in bmao based on their MCMC frequencies, relative to all models
%    encountered by bmao - see 'Notes' %  
%
%% NOTES:
%A call to bms with an MCMC sampler   uses a Metropolis-Hastings algorithm to sample through the model space - and the frequency of how often models are drawn converges to the distribution of their posterior marginal likelihoods. While sampling, each 'bma' object stores the best models encountered by its sampling chain with their marginal likelihood and their MCMC frequencies.
% pmp.bma then allows for comparing the posterior model probabilities (PMPs) for the two different methods, similar to plotConv. It calculates the PMPs based on marginal likelihoods (first column) and the PMPs based on MCMC frequencies (second column) for the best x models stored in the bma object.
% The correlation of the two columns is an indicator of how well the MCMC sampler has converged to the actual PMP distribution - it is therefore also given in the output of summary.bma.
% The second column thus shows the PMPs of the best models relative to all models the call to bms has sampled through (therefore typically the second column adds up to less than one). However, the first column only relates to the likelihoods of the best models, therefore it adds up to 1. (the marginal likelihoods for the models not retained in the best models cannot be known).
% The second column is slightly different in case the bms argument mcmc was set to mcmc="enumeration": In this case, the second column is also based on marginal likelihoods. The correlation between the two columns is therefore one. 
%
%% EXAMPLE:
% load datafls;
% mm=bms(datafls,[],[],20) %store only the best 20 models
% pmp_bma(mm)
%________________________________________________
%
%% SEE ALSO
% bms, plootConv for plotting something like pmp_bma
% 
% Equivalent function in R: pmp.bma
% Author: Stefan Zeugner 2010 
% Url: http://bms.zeugner.eu

if nargin<2; index_of_models=NaN; end;
if isempty(index_of_models); index_of_models=NaN; end;


    isit=is_bma(bmao,false);
    evalinR(['bmstmp= pmp.bma(bma', bmao.suffix, ')']);
    pmpstemp=rcell2mat(getRdata('bmstmp'));
    if any(isnan(index_of_models));
        index_of_models=1:size(pmpstemp,1);
    end;

    pmpstemp=pmpstemp(index_of_models,:);
    
    if nargout>0;
        pmps=pmpstemp;
    else
        pmpsdisp=c2cstr(pmpstemp);
        pmpsdisp=vertcat({'PMP (Exact)', 'PMP (MCMC)' }, pmpsdisp );
        disp(Cstr2format(pmpsdisp,1,0,3));
    end